/*******************************************************************************************/
/*                              file str_utils.c                                          */
/*                                                                                         */
/*           Copyright (C) S. Krishnan <sri_krishnan@yahoo.com>, 2000, 2001                */
/*                                                                                         */
/*     General string splitting and cleaning routines                                      */
/*                                                                                         */
/*     Version 0.4 dt. 11.12.2000                                                          */
/*     language: C                                                                         */
/*     Compiler: gcc                                                                       */
/*     Dependencies:  libc                                                                 */
/*******************************************************************************************/         




/*

This file comprises some general purpose string manipulation routines for Linux
written by S. Krishnan <sri_krishnan@yahoo.com>.  

It is free software; you can redistribute it and/or  modify it under the terms of 
the GNU Library General Public License (LGPL) as published by the 
Free Software Foundation; either version 2, or (at your option) any later version.  
You are free to link this code into any software that you write,
even proprietary, provided that due credit is given to the author and this copyright
message is displayed along with either this source file, or else directions to obtain it
as a download from the Internet or by any other means accessible to the public at large.

Please note that these routines are still a work in progess.

These routines are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Library General Public License along
with this file; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.

*/



#include <stdio.h>
#include <stdlib.h>


/* split_string revised Dec. 2000 */

/* 
 * split_string tokenizes a string passed to it by splitting it along a specified
 * delimiter character and returns the split pieces as a char ** pointer to several char arrays,
 * each of which contains one of the split pieces.  The end of the array of arrays is 
 * denoted by NULL, so that it is easy to use the return value in loops by simply checking
 * for a NULL value with a (if *ret_string_array==NULL) type of construct.
*/

char **split_string(char *string, char separator)
{
	char **str_stack, *split_ptr, *temp;
	int count=0, len=0, str_end=0; 

	split_ptr=string;
	while(*split_ptr++)
		if(*split_ptr == separator)
			count++;   // count how many pieces will result

	if((str_stack = (char **)calloc((count+1), sizeof(char *)))==NULL)
	   return NULL;
        /* allocate enough stack space to hold the broken pieces */

        if((temp = (char *)malloc(strlen(string)))==NULL)
	  return NULL;

	count=0;
	while(1)
		{
		if((*string==separator) || (*string=='\0')) 
			{
			if(!str_end)
				{
				*temp='\0';
				temp-=len;		
				if((*str_stack = (char *)malloc(len+1))==NULL)
				  return NULL;
				strcpy(*str_stack, temp);
				str_stack++;
				count++;
				len=0;
				}
			else	
				break;
			}
		else
		{
			*temp=*string;
			temp++;
			len++;
			}
			if(*string=='\0')
				str_end=1;
			else
				string++;
		}

	if((*str_stack=(char *)malloc(sizeof(char *)))==NULL)
	  return NULL;
	*str_stack = (char *)0;	 // so that we can recog. end of stack
	free(temp);
	return(str_stack-count);
}


/*---------------------------------------------------------------------------------*/


char *strclean(char *string)
{
  char *instring, *string_base, *mid_string;
  int string_state=0;

  instring=strdup(string);
  string_base=instring;

  while(*instring)
    {
      switch (*instring)
	{
	case '\t':
	case '\n':
	case '\r':
	case ' ':                        
	  if(!string_state)
	    instring++;	  
	  if(string_state==1)         
	    {
	      if(!*(instring+1))
		string_state=2;
	      else
		{
		  mid_string=instring;     /* save the current string position */
		  while(*instring)
		    {
		      *instring = *(instring+1);
		      instring++;
		    }
		  instring=mid_string;
		  instring++;
		}
	    }
	  if(string_state==2)             /* the end of the string is upon us */
	      *instring='\0';
	  break;
	case '\0':
	  break;
	default:
	  if(string_state==0)
	    {
	      string_state=1;
	      string_base=instring;
	    }
	  instring++;
	  break;
	}    
    }
  return string_base;
}






















